/*======================================================================*/
/* gpfunc.h: bibliothque de fonctions gnrales						*/
/*======================================================================*/
/* COPYRIGHT 1993 Jean-Pierre MENICUCCI - Tous droits rservs			*/
/*======================================================================*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*//////////////////////////////////////////////////////////////////////*/
/* fonctions de messages												*/
/*//////////////////////////////////////////////////////////////////////*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Les messages utiliss dans ces fonctions sont identifis par leur	*/
/* numros dans les ressources de la bibliothque ou de l'appelant.		*/
/*																		*/
/* Cela permet de maintenir l'ensemble des messages dans les ressources	*/
/* sans en avoir aucun dans le code, et de ne pas avoir  grer le		*/
/* chargement des messages dans les variables de l'appelant.			*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*======================================================================*/
/* MesBox: affichage d'une bote de message								*/
/* retour:																*/
/*		identificateur du bouton actionn								*/
/*======================================================================*/
/* Le message affich est identifi par son numro dans les ressources	*/
/* de l'instance.														*/
/*======================================================================*/

int FAR PASCAL MesBox (
		HANDLE hInstance,		// instance contenant le message
		LPSTR lpszCaption,		// titre du dialogue
		WORD wMessage,			// numro de message
		WORD wFlags				// boutons de fin du dialogue
		);

/*======================================================================*/
/* StrMesBox: affichage d'une bote de message							*/
/* retour:																*/
/*		identificateur du bouton actionn								*/
/*======================================================================*/
/* Le message affich est cr d'aprs deux constituants:				*/
/*																		*/
/* 1 -	Un message de formattage identifi par son numro dans les		*/
/*		ressources de l'instance, contenant un %s pour indiquer			*/
/*		l'endroit de l'insertion du deuxime constituant.				*/
/*																		*/
/* 2 -	Un lment intgr au niveau du %s et le remplaant.			*/
/*======================================================================*/

int FAR PASCAL StrMesBox (
		HANDLE hInstance,		// instance contenant le message
		LPSTR lpszCaption,		// titre du dialogue
		WORD wMessage,			// numro de message de formattage
		LPSTR lpszString,		// paramtre  intgrer
		WORD wFlags				// boutons de fin du dialogue
		);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*//////////////////////////////////////////////////////////////////////*/
/* fonctions de manipulation de chanes de caractres					*/
/*//////////////////////////////////////////////////////////////////////*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*======================================================================*/
/* wCpy: copie d'une chane sur une autre								*/
/* retour:																*/
/*		longueur de la chane aprs copie								*/
/*======================================================================*/
/* Les fonctions wCpy et wCat sont utilises en lieu et place des fonc-	*/
/* tions standard Windows lstrcpy et lstrcat. Elles ne grent pas d'oc-	*/
/* tet nul de fin de chane, et wCat reoit en entre la longueur ini-	*/
/* tiale de la chane  copier, ce qui vite d'avoir  la recalculer	*/
/*  chaque agrandissement.												*/
/*======================================================================*/

WORD FAR PASCAL wCpy (
		LPSTR pOut,				// chane en sortie
		LPSTR pIn,				// chane en entre
		WORD wLen				// longueur  copier
		);

/*======================================================================*/
/* wCat: concatnation d'une chane dans une autre						*/
/* retour:																*/
/*		longueur de la chane aprs concatnation						*/
/*======================================================================*/

WORD FAR PASCAL wCat (
		LPSTR pOut,				// chane en sortie
		WORD wOffset,			// longueur avant concatnation
		LPSTR lpIn,				// chane en entre
		WORD wLen				// longueur  copier
		);

/*======================================================================*/
/* hCpy: copie d'une chane sur une autre								*/
/* retour:																*/
/*		longueur aprs copie											*/
/*======================================================================*/
/* Les fonctions hCpy et hCat sont une extention des fonctions standard	*/
/* Windows lstrcpy et lstrcat.  Elles utilisent des pointeurs maximaux,	*/
/* permettant  le travail sur  des  chanes  de plus  de 1 mga-octets,	*/
/* reoivent  en paramtres  la taille de  la chane rceptrice, ce qui	*/
/* permet  d'viter les  dbordements  intempestifs, et hCat  reoit en	*/
/* entre la longueur  initiale de de la chane  copier, ce qui vite	*/
/* d'avoir  la recalculer  chaque agrandissement.						*/
/*======================================================================*/

long FAR PASCAL hCpy (
		char huge *pOut,		// chane en sortie
		long mOut,				// taille de la chane en sortie
		char huge *pIn,			// chane en entre
		long lIn				// longueur  copier
		);

/*======================================================================*/
/* hCat: concatnation d'une chane sur une autre						*/
/* retour:																*/
/*		longueur aprs concatnation									*/
/*======================================================================*/

long FAR PASCAL hCat (
		char huge *pOut,		// chane en sortie
		long mOut,				// taille de la chane en sortie
		long lOut,				// longueur avant concatnation
		char huge *pIn,			// chane en entre
		long lIn				// longueur  copier
		);

/*======================================================================*/
/* hCmp: comparaison de deux chanes									*/
/* retour:																*/
/*		-1L		chane 1 < chane 2										*/
/*		0L		chane 1 == chane 2									*/
/*		1L		chane 1 > chane 2										*/
/*======================================================================*/

long FAR PASCAL hCmp (
		char huge *In1,			// chane 1
		char huge *In2,			// chane 2
		long lLen				// longueur  comparer
		);

/*======================================================================*/
/* Lengthen: ajout d'espaces  une chane								*/
/* retour:																*/
/*		vide															*/
/*======================================================================*/

void FAR PASCAL Lengthen (
		LPSTR pOut,				// chane  allonger
		long lOut,				// longueur actuelle
		long mOut				// taille
		);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*//////////////////////////////////////////////////////////////////////*/
/* fonctions de transcodage de chanes de caractres					*/
/*//////////////////////////////////////////////////////////////////////*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*======================================================================*/
/* aschex: transformation de la reprsentation hexadcimale				*/
/*		d'une chane d'octets en chane d'octets						*/
/* retour:																*/
/*		longueur de la chane en sortie									*/
/*======================================================================*/

long FAR PASCAL aschex (
		unsigned char far *pIn,		// reprsentation hexadcimale
		long lIn,					// longueur
		unsigned char far *pOut		// chane d'octets
		);

/*======================================================================*/
/* hexasc: transformation d'une chane d'octets en sa reprsentation	*/
/*		hexadcimale imprimable											*/
/* retour:																*/
/*		longueur de la chane en sortie									*/
/*======================================================================*/

long FAR PASCAL hexasc (
		unsigned char far *pIn,		// chane d'octets
		long lIn,					// longueur
		unsigned char far *pOut		// reprsentation hexadcimale
		);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*//////////////////////////////////////////////////////////////////////*/
/* fonctions de dlimitation dans les chanes de caractres				*/
/*//////////////////////////////////////////////////////////////////////*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Ces fonctions facilitent la lecture de fichiers de paramtres dont	*/
/* la syntaxe est constitue de paramtres spars par des				*/
/* dlimiteurs.															*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*======================================================================*/
/* Point: pointer le caractre suivant un dlimiteur					*/
/* retour:														 		*/
/*		longueur saute											 		*/
/*======================================================================*/
/* Point utilise en entre un pointeur sur un pointeur sur une chane	*/
/* de caractres. A partir de la position initiale, on se dplace		*/
/* jusqu'aprs le dlimiteur spcifi. Le pointeur sur la chane		*/
/* de caractres est mis  jour en consquence et le nombre de			*/
/* caractres sauts est retourn.										*/
/*======================================================================*/

long FAR PASCAL Point (
		char huge * far *pIn,	// chane en entre
		char cDel				// dlimiteur
		);

/*======================================================================*/
/* Pull: extraire une sous-chane dlimite								*/
/* retour:																*/
/*		longueur extraite												*/
/*======================================================================*/
/* Pull fonctionne de la mme faon que Point, mais en outre stocke les	*/
/* caractres contenus entre la position initiale du pointeur et le		*/
/* dlimiteur dans une chane rceptrice. Si la chane rceptrice		*/
/* est trop courte, les caractres excdentaires ne sont pas pris		*/
/* en compte.															*/
/*======================================================================*/

long FAR PASCAL Pull (
		char huge * far *pIn,	// chane en entre
		char huge *pOut,		// chane en sortie
		long mOut,				// taille en sortie
		char cDel				// dlimiteur
		);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*//////////////////////////////////////////////////////////////////////*/
/* fonctions de manipulation de fichiers								*/
/*//////////////////////////////////////////////////////////////////////*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*======================================================================*/
/* FileLength: dtermination de la taille d'un fichier					*/
/* retour:																*/
/*		taille du fichier												*/
/*======================================================================*/

long FAR PASCAL FileLength (
		HFILE hFile				// poigne fichier
		);

/*======================================================================*/
/* SplitPath: obtention des lments constitutifs d'un nom de fichier	*/
/* retour:																*/
/*		vide															*/
/*======================================================================*/
/* SplitPath reoit en entre une chane de caractres contenant un nom */
/* de fichier complet, avec spcification du lecteur et des rpertoires	*/
/* intermdiaires.														*/
/*======================================================================*/

void FAR PASCAL SplitPath (
		LPSTR lpszPath,			// nom de fichier complet
		int far *iPath,			// indice de la chane de rpertoires
		int far *iRadical,		// indice du radical du nom de fichier
		int far *iSuffix		// indice du suffixe du nom de fichier
		);

/*======================================================================*/
/* MakePath: cration de la chane de rpertoires menant  un fichier	*/
/* retour:																*/
/*		0	ok															*/
/*		#0	numro d'erreur systme										*/
/*======================================================================*/

int FAR PASCAL MakePath (
		LPSTR AppName,			// nom de l'application
		LPSTR DirPath			// arborescence  crer
		);

/*======================================================================*/
/* CopyFile: copie d'un fichier sur un autre							*/
/* retour:																*/
/*		0	ok															*/
/*		#0	ko															*/
/*======================================================================*/
/* Le buffer est fourni par l'appelant. Une taille de buffer de 8192	*/
/* octets semble optimale.												*/
/*======================================================================*/

int FAR PASCAL CopyFile (
		LPSTR AppName,			// nom de l'application
		LPSTR DstPath,			// chemin du fichier destination
		LPSTR SrcPath,			// chemin du fichier source
		LPSTR pBuf,				// buffer
		int mBuf				// taille buffer
		);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*//////////////////////////////////////////////////////////////////////*/
/* fonctions de manipulation d'un fichier profil						*/
/*//////////////////////////////////////////////////////////////////////*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Ces fonctions compltent les  fonctions standard Windows:			*/
/* - GetProfileBuffer permet de lire tout le profil en mmoire			*/
/* - FindProfileSection permet de localiser le dbut d'une section		*/
/* - GetProfileLine permet de lire la section ligne  ligne, ce qui est	*/
/*   utile lorsque les valeurs des cls (membres gauches des lignes de	*/
/*   la section) ne sont pas connues  priori.							*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*======================================================================*/
/* GetProfileBuffer: obtention du contenu d'un fichier profil			*/
/* retour:																*/
/*		0						ok										*/
/*		-1						ko										*/
/*======================================================================*/
/* GetProfileBuffer alloue de la mmoire, y stocke le fichier profil,	*/
/* et retourne une poigne et un pointeur sur cette mmoire, pour		*/
/* disponibilit pour et gestion par l'appelant.						*/
/*======================================================================*/

int FAR PASCAL GetProfileBuffer (
		LPSTR AppName,			// nom de l'application
		LPSTR ProPath,			// chemin du fichier profil
		HGLOBAL far *hBuf,		// poigne mmoire alloue
		char huge * far *pBuf	// adresse mmoire alloue
		);

/*======================================================================*/
/* FindProfileSection: trouver le dbut d'une section					*/
/* retour:																*/
/*		0		ok														*/
/*		-1		ko - section non trouve								*/
/*======================================================================*/
/* FindProfileSection localise le dbut d'une section dans un profil.	*/
/* lCharBeg retourn pointe sur le premier caractre de la premire		*/
/* ligne de la section.													*/
/*======================================================================*/

int FAR PASCAL FindProfileSection (
		LPSTR AppName,			// nom de l'application
		LPSTR SecName,			// nom de la section
		BOOL bRequired,			// section requise
		char huge *pBuf,		// adresse mmoire du profil
		long far *lCharBeg		// 1ere ligne de la section
		);

/*======================================================================*/
/* GetProfileLine: lecture d'une ligne de profil						*/
/* retour:																*/
/*		dplacement jusqu' la ligne suivante							*/
/*======================================================================*/
/* GetProfileLine extrait la ligne courante de profil. Si la ligne		*/
/* contient un dlimiteur =, elle est scinde en une partie gauche		*/
/* et une partie droite. Sinon, la partie gauche contient la ligne		*/
/* entire.																*/
/*																		*/
/* La valeur longue retourne par la fonction prcise le nombre de		*/
/* caractres  sauter pour trouver le dbut de la ligne suivante.		*/
/* Cette valeur est nulle si la fin du profil est atteinte.				*/
/*======================================================================*/

long FAR PASCAL GetProfileLine (
		char huge *pBuf,		// adresse mmoire
		long far *lLef,			// longueur partie gauche
		char huge * far *pRig,	// pointeur partie droite
		long far *lRig			// longueur partie droite
		);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*//////////////////////////////////////////////////////////////////////*/
/* fonctions de manipulation de bitmaps									*/
/*//////////////////////////////////////////////////////////////////////*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*======================================================================*/
/* GetBitmap: cration d'une bitmap  partir d'un fichier bitmap		*/
/* retour:																*/
/*		# -1	poigne sur la bitmap									*/
/*		-1		ko														*/
/*======================================================================*/
/* GetBitmap cre et charge une Bitmap  partir d'un fichier Bitmap.	*/
/* GetBitmap retourne une poigne Bitmap directement utilisable par		*/
/* l'appelant.															*/
/*======================================================================*/

HBITMAP FAR PASCAL GetBitmap (
		LPSTR AppName,			// nom de l'application
		LPSTR FilePath,			// chemin du fichier bitmap
		HDC hDC					// contexte priphrique
		);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*//////////////////////////////////////////////////////////////////////*/
/* fonctions de chiffrement des (data encryption standard)				*/
/*//////////////////////////////////////////////////////////////////////*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*======================================================================*/
/* desc: codage des														*/
/* retour:																*/
/*		vide															*/
/*======================================================================*/

void FAR PASCAL desc (
		unsigned char far *in,	// 64 bits  chiffrer
		unsigned char far *out,	// 64 bits chiffrs
		unsigned char far *key	// cl sur 64 bits
		);

/*======================================================================*/
/* desd: decodage des													*/
/* retour:																*/
/*		vide															*/
/*======================================================================*/

void FAR PASCAL desd (
		unsigned char far *in,	// 64 bits  dchiffrer
		unsigned char far *out,	// 64 bits dchiffrs
		unsigned char far *key	// cl sur 64 bits
		);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*//////////////////////////////////////////////////////////////////////*/
/* fonctions de date et d'heure											*/
/*//////////////////////////////////////////////////////////////////////*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*======================================================================*/
/* Date: renvoi de la date et de l'heure								*/
/* retour:																*/
/*		vide															*/
/*======================================================================*/
/* Date renvoie la date et l'heure en respectant les spcificits		*/
/* nationales dfinies dans la configuration. Le format retourn est:	*/
/* - MM/DD/YY HH:MM pour les Etats-Unis (mois, jour, anne)				*/
/* - DD/MM/YY HH:MM pour l'Europe (jour, mois, anne)					*/
/* - YY/MM/DD HH:MM pour le Japon (anne, mois, jour)					*/
/*======================================================================*/

void FAR PASCAL Date (
		LPSTR pDate				// chane de 19 caractres
		);

/************************************************************************/
/* I D E N T I F I C A T E U R S    D E S    R E S S O U R C E S		*/
/************************************************************************/

#define GPF_ERROPEN		1		// Erreur d'ouverture de "%s" !
#define GPF_ERRREAD		2		// Erreur de lecture de "%s" !
#define GPF_ERRCREATE	3		// Erreur de cration de "%s" !
#define GPF_ERRWRITE	4		// Erreur d'criture de "%s" !
#define GPF_ERRALLOC	5		// %s: Allocation mmoire impossible !
#define GPF_ERRLOCK		6		// %s: Accs mmoire impossible !
#define GPF_ERRDIB		7		// %s: Cration de la bitmap impossible !
#define GPF_ERRSEC		8		// Section [%s] inexistante !

