{**********************************************************}
{                                                          }
{                   Projet SoftOptique                     }
{          Drivers d'Evnements : Souris, Clavier          }
{         Init du mode graphique, Load des dessins         }
{    Accs aux Rpertoires, Calcul de la configuration     }
{                                                          }
{   Debut du projet : 10.04.93                             }
{   Mise A Jour     : 04.05.93                             }
{                                                          }
{**********************************************************}

UNIT Drivers;

INTERFACE

Const
   { Nombre de motifs graphiques utiliss }
     NbIco        = 13;                     { Icones 20*20                   }
     NbIco2       = 12;                     { Icones 32*32                   }
     NbMot        = 16;                     { Motifs de taille quelconque    }
     NbMouses     = 12;                     { Nombre de curseurs souris      }
     NbChiffres   = 18;                     { Nombre de motifs digitaux      }
     WorldX       : Array[-12..12,1..2] Of Integer =      { Coordonnes X    }
     ((000,019),(000,023),(007,031),(017,036),(020,041),  { des motifs pour  }
      (028,049),(034,055),(040,062),(047,067),(049,073),  { les fuseaux      }
      (062,081),(068,090),(069,096),(074,104),(081,106),  { horaires         }
      (091,116),(097,118),(103,126),(109,131),(115,140),
      (117,147),(129,152),(132,161),(140,161),(000,161));
   { Constantes types d'vnements }
     evNothing    = $0000;
     evMouseLDown = $0001;
     evMouseLUp   = $0002;
     evMouseRDown = $0004;
     evMouseRUp   = $0008;
     evMouseMove  = $0010;
     evMouseAuto  = $0020;
     evMouse      = $003F;
     evMouseAct   = $000F;
     evKeyDown    = $0040;
     evToggle     = $0080;
     evCommand    = $0100;
     evAuto       = $0200;
     evTime       = $0400;
  { Masques Touches Toggle }
     ToggleCtrl   = $04;
     ToggleAlt    = $08;
     ToggleNum    = $20;
     ToggleCaps   = $40;
     ToggleIns    = $80;
  { Noms des divers processeurs }
     NomProcess   : Array[0..7] of String[12] =
      ('Intel 8088','Intel 8086','Nec V20','Nec V30',
       'Intel 80188','Intel 80186','Intel 80286','i386 ou i486');
  { Noms des jours de la semaine }
     NomJour      : Array[0..6] Of String[8] =
     ('Dimanche','Lundi','Mardi','Mercredi','Jeudi','Vendredi','Samedi');
  { Noms des mois }
     NomMois      : Array[1..12] Of String[9] =
     ('Janvier','Frvrier','Mars','Avril','Mai','Juin','Juillet','Aot',
      'Septembre','Octobre','Novembre','Dcembre');
  { Type de ligne pour animation }
     ZoneDefLn : Array[1..4] Of Word = ($CCCC,$9999,$3333,$6666);
  { Dfinition de PI }
     PI : Real = 3.14159265359;

Type TPoint   = Record                           { Type de base pour toutes }
                  X,Y : Integer;                 { les spcifications de    }
                End;                             { Position cran           }
     TRay     = Record                           { Rayon lumineux           }
                  X,Y,A : Real;
                End;
     TEvent   = Record                           { Type vnement           }
                  What  : Word;                  { Nature de l'vnement    }
                  case Word of                   { Puis le contenu          }
                    evNothing: ();               { Evnement vide           }
                    evMouse: (                   { Evnement souris         }
                      LButton: Boolean;          { Etat bouton de gauche    }
                      RButton: Boolean;          { Etat bouton de droite    }
                      Where  : TPoint);          { Position de la souris    }
                    evKeyDown: (                 { Evnement clavier        }
                      C1,C2 : Char);             { Code normal et tendu    }
                    evCommand: (                 { Commande interne         }
                      Command: Word;             { No de la commande        }
                      case Word of               { Champ info :             }
                        0:(InfoByte : Byte);     { de type Byte             }
                        1:(InfoInt  : Integer);  { de type Integer          }
                        2:(InfoPtr  : Pointer)); { de type Pointer          }
                End;
     PDskItem = ^TDskItem;
     PLecteur = ^TLecteur;
     TDskItem = Record                           { Elment de directory     }
                  Suiv,Prec : PDskItem;          { D'un disque : chanage   }
                  Liste     : PDskItem;          { Sous-rpertoire ventuel }
                  Nom       : String[12];        { Nom de l'entre directory}
                  Attr      : Byte;              { Attribut                 }
                End;
     TLecteur = Record                           { Chainage pour la liste   }
                  Suiv   : PLecteur;             { No : Id lecteur (1=A...) }
                  No,Tpe : Byte;                 { Tpe : type 1=disque dur  }
                End;                             { 2=3.5"  3=5.25"          }
     DskItemTb = Array[1..255] Of PDskItem;
     PMouseAspect = ^TMouseAspect;
     TMouseAspect = Record
                      DX,DY : Integer;
                      cAND  : Array[1..16] Of Word;
                      cOR   : Array[1..16] Of Word;
                    End;
     PZoom    = ^TZoom;
     TZoom    = Record
                  Suiv        : PZoom;
                  X1,Y1,X2,Y2 : Real;
                  Pas         : Real;
                  Gril        : Boolean;
                End;

Var Lecteurs  : PLecteur;              { Tte de la liste des lecteurs       }
    DskRoot   : PDskItem;              { Tte de l'arbre du directory en RAM }
    TypePC    : String[2];             { Type du PC ('AT','XT','PC')         }
    Processor : String[12];            { Nom du processeur                   }
    DosVer1   : Byte;                  { Version du DOS                      }
    DosVer2   : Byte;
    GrCarte   : Integer;               { Carte et mode graphique  utiliser  }
    GrMode    : Integer;
    AppPath   : String;                { Chemin de Optique.EXE               }
    Toggle    : Byte;                  { Etat des touches Toggle             }
    MouseX    : Integer;
    MouseY    : Integer;               { Position courante de la souris      }
    Icones    : Array[1..NbIco]  Of Pointer;
    Icones2   : Array[1..NbIco2] Of Pointer;
    Motifs    : Array[1..NbMot]  Of Pointer;
    World     : Array[-12..12] Of Pointer;
    MotifsC   : Array[1..NbMot] Of TPoint;
    Chiffres  : Array[0..NbChiffres] Of Pointer;
    Mouses    : Array[1..NbMouses] Of TMouseAspect;
    Heure     : Integer;
    Minute    : Integer;
    Seconde   : Integer;
    MouseCur  : PMouseAspect;
    Zooms     : PZoom;
    ForceEGA  : Boolean;
    ForceVGA  : Boolean;
    NoPre     : Boolean;

{ Noms des divers pointeurs souris, Icones, Icones2, Motifs }
{ pour claret du listing                                   }
Const
    MousePointer : PMouseAspect = @Mouses[1];
    MouseWait    : PMouseAspect = @Mouses[2];
    MouseText    : PMouseAspect = @Mouses[3];
    MouseDown    : PMouseAspect = @Mouses[4];
    MouseUp      : PMouseAspect = @Mouses[5];
    MouseZoom    : PMouseAspect = @Mouses[6];
    MouseRight   : PMouseAspect = @Mouses[7];
    MouseLeft    : PMouseAspect = @Mouses[8];
    MouseCroix   : PMouseAspect = @Mouses[9];
    MouseLentil  : PMouseAspect = @Mouses[10];
    MouseMiroir  : PMouseAspect = @Mouses[11];
    MouseDiaph   : PMouseAspect = @Mouses[12];
    icZoomPlus   = 1;
    icZoomMoins  = 2;
    icPointer    = 3;
    icLentil     = 4;
    icMiroir     = 5;
    icDiaphragme = 13;
    icPoubVide   = 6;
    icObjet      = 7;
    icQuit       = 11;
    icDisk       = 8;
    icHelp       = 10;
    icConfig     = 12;
    icPoubPleine = 9;

Procedure DriversInit;
Procedure DriversDone;
{ Routines d'acquisition des vnements }
Procedure GetMouseEvent(Var Event : TEvent);
Procedure GetKeyboardEvent(Var Event : TEvent);
Procedure GetTimeEvent(Var Event : TEvent);
{ Routines de manipulation souris }
Procedure ChangeMouseAspect(M : PMouseAspect);
Procedure MouseClipON(X1,Y1,X2,Y2 : Integer);
Procedure MouseClipOFF;
Procedure MouseHide;
Procedure MouseShow;
Procedure MouseLimits(X1,Y1,X2,Y2 : Integer);
Procedure MouseMoveTo(X,Y : Integer);
{ Routines de manipulation des directories }
Procedure EnterDiskDir;
Procedure DoneDskItems;
Procedure InitDskItems;
Procedure GetDir(D: Byte; var S: string);
Procedure ChDir(S : String);
{ Routines de manipulation du Zoom }
Procedure Zoom(X1,Y1,X2,Y2,P : Real ; G : Boolean);
Procedure UnZoom;
Function  Ex(X : Real) : Integer;
Function  Ey(Y : Real) : Integer;
Function  Rx(X : Integer) : Real;
Function  Ry(Y : Integer) : Real;
{ Routines graphiques }
Procedure Outline(X,Y,L,H : Integer ; C1,C2,C3,C4 : Byte);
{ Routine sonore }
Procedure BeepWrong;
{ Routines diverses }
Function  MaxByte(A,B : Byte) : Byte;
Function  MinByte(A,B : Byte) : Byte;
Function  MaxWord(A,B : Word) : Word;
Function  MinWord(A,B : Word) : Word;
Function  MaxReal(A,B : Real) : Real;
Function  MinReal(A,B : Real) : Real;
Function  Tan(A : Real) : Real;
Function  GetLongTime : LongInt;
Function  UpStr(S : String) : String;

IMPLEMENTATION

{$V-}     { Pour les assignations entre chanes de longueurs diffrentes }
{$S-}     { Pour la rapidit du Quick-Sort                               }

USES Crt,Dos,Graph,Disk;

Var Reg : Registers;

{ ==== Module : Heure ==== }

Procedure GetTimeEvent(Var Event : TEvent);
Var S100  : Word;
    H,M,S : Word;
Begin
  Event.What:=evNothing;
  GetTime(H,M,S,S100);
  If (S<>Seconde) Then
  Begin
    Seconde:=S;
    Heure:=H;
    Minute:=M;
    Event.What:=evTime;
  End;
End;

{ ==== Module : Souris ==== }


Var MouseStatus     : Integer;         { Etat des boutons                    }
    ClipX1,ClipY1   : Integer;         { Zone o la souris s'efface          }
    ClipX2,ClipY2   : Integer;
    ClipNiveau      : Integer;         { Niveau d'imbrication des Clip       }

Procedure MouseMoveTo(X,Y : Integer); assembler;
asm
    MOV   AX,4
    MOV   CX,X                 { Place le curseur au milieu de l'cran  X,Y }
    MOV   DX,Y
    INT   $33
end;

Procedure MouseLimits(X1,Y1,X2,Y2 : Integer); assembler;
asm
    MOV   AX,7               { Fixe la zone de dplacement X de X1  X2 }
    MOV   CX,X1
    MOV   DX,X2
    INT   $33
    MOV   AX,8               { Fixe la zone de dplacement Y de Y1  Y2 }
    MOV   CX,Y1
    MOV   DX,Y2
    INT   $33
end;

Procedure MouseHide;
Begin
  If ClipNiveau=0 Then
  asm
    MOV  AX,2
    INT  $33
  End;
  Inc(ClipNiveau);
end;

Procedure MouseShow;
Begin
  Dec(ClipNiveau);
  If ClipNiveau=0 Then
  asm
    MOV  AX,1
    INT  $33
  End;
end;

Procedure ChangeMouseAspect(M : PMouseAspect);
Begin
  If M<>MouseCur Then
  Begin
    Reg.AX:=9;
    Reg.BX:=M^.DX;
    Reg.CX:=M^.DY;
    Reg.ES:=Seg(M^.cAND);
    Reg.DX:=Ofs(M^.cAND);
    Intr($33,Reg);
    MouseCur:=M;
  End;
End;

Procedure MouseClipON(X1,Y1,X2,Y2 : Integer);
  Function Min(a,b : Integer) : Integer;
  Begin
    If a<b Then Min:=a Else Min:=b;
  End;
  Function Max(a,b : Integer) : Integer;
  Begin
    If a<b Then Max:=b Else Max:=a;
  End;
Begin
  If ClipNiveau=0
  Then
    Begin
      ClipX1:=X1;             { Si c'est le premier appel, le Clip courant   }
      ClipY1:=Y1;             { est gal  celui qui est demand             }
      ClipX2:=X2;
      ClipY2:=Y2;
    End
  Else
    Begin
      ClipX1:=Min(X1,ClipX1); { Si ce n'est pas le premier appel, le Clip    }
      ClipX2:=Max(X2,ClipX2); { Courant vaut la runion de l'ancien clip et  }
      ClipY1:=Min(Y1,ClipY1); { de celui demand                             }
      ClipY2:=Max(Y2,ClipY2);
    End;
  asm
    MOV   AX,$10              { Dfinit un rectangle Clip pour le            }
    MOV   CX,ClipX1           { Curseur souris. Le curseur disparait         }
    MOV   DX,ClipY1           { S'il est dans la zone Clip                   }
    MOV   SI,ClipX2
    MOV   DI,ClipY2
    INT   $33
    CLI                       { Interdit le mouvement pendant le trac       }
  end;
  Inc(ClipNiveau);            { Augmente le niveau d'imbrication des appels  }
End;

Procedure MouseClipOFF;
Begin
  Dec(ClipNiveau);            { Diminue le niveau d'imbrication des appels   }
  if ClipNiveau=0 Then        { Si on est arriv au dernier niveau           }
  asm
    MOV  AX,1                 { On R-Affiche le curseur souris              }
    INT  $33                  { Et annule l'effet du Clip                    }
    STI                       { On autorise le mouvement                     }
  end;
End;

Procedure MouseInit;
Begin
  asm
    MOV   AX,0               { Initialise le pilote souris           }
    INT   $33
  end;
  MouseLimits(0,0,GetMaxX,GetMaxY);
  MouseMoveTo(GetMaxX div 2, GetMaxY div 2);
  MouseShow;
  ClipNiveau:=0;
  MouseCur:=NIL;
  ChangeMouseAspect(MousePointer);
End;

Procedure MouseDone; assembler;
asm
  MOV  AX,0
  INT  $33
End;

Procedure GetMouseEvent(Var Event : TEvent);
Begin
  Reg.AX:=3;
  Intr($33,Reg);
  Event.LButton:=(Reg.BX AND 1)<>0;
  Event.RButton:=(Reg.BX AND 2)<>0;
  Event.Where.X:=Reg.CX;
  Event.Where.Y:=Reg.DX;
  Event.What:=evNothing;
  If (Reg.CX<>MouseX) OR (Reg.DX<>MouseY) Then
  Begin
    MouseX:=Reg.CX;
    MouseY:=Reg.DX;
    Event.What:=evMouseMove;
  End;
  If (MouseStatus AND 1)<>(Reg.BX AND 1) Then
    If (MouseStatus AND 1)<>0
    Then
      Begin
        MouseStatus:=MouseStatus AND $FFFE;
        Event.What:=evMouseLUp;
      End
    Else
      Begin
        MouseStatus:=MouseStatus OR 1;
        Event.What:=evMouseLDown;
      End;
  If (MouseStatus AND 2)<>(Reg.BX AND 2) Then
    If (MouseStatus AND 2)<>0
    Then
      Begin
        MouseStatus:=MouseStatus AND $FFFD;
        Event.What:=evMouseRUp;
      End
    Else
      Begin
        MouseStatus:=MouseStatus OR 2;
        Event.What:=evMouseRDown;
      End;
End;

{ === Module : Clavier === }

Var SysToggle : Byte Absolute $0040:$0017;

Procedure GetKeyboardEvent(Var Event : TEvent);
Begin
  Event.What:=evNothing;
  If (SysToggle<>Toggle) Then
  Begin
    Toggle:=SysToggle;
    Event.What:=evToggle;
    Exit;
  End;
  If Keypressed Then
  Begin
    Event.What:=evKeyDown;
    Event.C2:=#0;
    Event.C1:=ReadKey;
    If Event.C1=#0 Then Event.C2:=ReadKey;
    Exit;
  End;
End;

Procedure KeyboardInit;
Begin
  SysToggle:=SysToggle OR ToggleIns;{ Fixe le mode Insertion par dfaut      }
  Toggle:=Not SysToggle;            { Avec cette valeur, l'tat sera affich }
                                    { du dbut correctement                  }
End;

Procedure KeyboardDone;
Begin
End;

{ ==== Module : Accs disque ==== }

Var DiskInitPath : String;
    QSortTab     : DskItemTb;

{ GetDir et ChDir sont modifies ici pour grer des Path se terminant }
{ toujours par '\'                                                    }

Procedure GetDir(D: Byte; var S: string);
Begin
  {$I-} System.GetDir(D,S); {$I+}
  If S[Length(S)]<>'\' Then S:=S+'\';
End;

Procedure ChDir(S : String);
Begin
  if Pos('\',S)<>Length(S) Then S:=Copy(S,1,Length(S)-1);
  {$I-} System.ChDir(S); {$I+}
End;

procedure Sort(l, r: Integer);
var
  i, j : integer;
  x, y : TDskItem;
begin
  i := l; j := r; x := QSortTab[(l+r) DIV 2]^;
  repeat
    while QSortTab[i]^.Nom < x.Nom do i := i + 1;
    while x.Nom < QSortTab[j]^.Nom do j := j - 1;
    if i <= j then
    begin
      y := QSortTab[i]^;
      QSortTab[i]^.Nom  :=QSortTab[j]^.Nom;
      QSortTab[i]^.Attr :=QSortTab[j]^.Attr;
      QSortTab[i]^.Liste:=QSortTab[j]^.Liste;
      QSortTab[j]^.Nom  :=y.Nom;
      QSortTab[j]^.Attr :=y.Attr;
      QSortTab[j]^.Liste:=y.Liste;
      i := i + 1; j := j - 1;
    end;
  until i > j;
  if l < j then Sort(l, j);
  if i < r then Sort(i, r);
end;

Procedure QSort(Item : PDskItem);
Var i : Byte;
begin {QuickSort};
  i:=0;                { Prpare le tableau QSortTab }
  While Item<>NIL do
  Begin
    Inc(i);
    QSortTab[i]:=Item; { Stocke l'adresse de l'item courant }
    Item:=Item^.Suiv;
  End;
  Sort(1,I);           { I Vaut ici le nombre d'items dans QSortTab }
end;

Procedure InitDskItems;
Var S : SearchRec;
    M : PMouseAspect;
  Function Cherche(Chemin : String) : PDskItem;
  Var I,I2 : PDskItem;
      P    : Byte;
      Temp : String;
  Begin
    I:=NIL;I2:=NIL;
    FindFirst(Chemin+'\*.*',Directory+Archive,S);
    While DosError=0 do
    Begin
      If (S.Name<>'.') AND (S.Name<>'..') Then
      Begin
        I2:=New(PDskItem);
        I2^.Suiv:=NIL;
        I2^.Prec:=I;
        I2^.Liste:=NIL;
        If I<>NIL Then I^.Suiv:=I2;
        I2^.Nom:=S.Name;
        I2^.Attr:=S.Attr AND Directory;
        If (I2^.Attr AND Directory)=0 Then
        Begin
          P:=Pos('.',I2^.Nom);
          If P=0 Then Begin I2^.Nom:=I2^.Nom+'.'; P:=Length(I2^.Nom); End;
          Temp:=Copy(I2^.Nom,1,P-1);
          I2^.Nom:=Copy(Temp+'        ',1,8)+Copy(I2^.Nom,P,4);
        End;
        I:=I2;
      End;
      FindNext(S);
    End;
    { Rcursion pour aller chercher les sous-rpertoires }
    While (I<>NIL) do
    Begin
      I2:=I;
      If (I2^.Attr AND Directory)<>0 Then
        I2^.Liste:=Cherche(Chemin+'\'+I2^.Nom);
      I:=I^.Prec;
    End;
    QSort(I2);
    Cherche:=I2;
  End;
Begin
  M:=MouseCur;
  ChangeMouseAspect(MouseWait);
  DskRoot:=Cherche('');
  ChangeMouseAspect(M);
End;

Procedure DoneDskItems;
  Procedure Libere(I : PDskItem);
  Var It,It2 : PDskItem;
  Begin
    It:=I;
    While (It<>NIL) do
    Begin
      It2:=It^.Suiv;
      If It^.Liste<>NIL Then Libere(It^.Liste);
      Dispose(It);
      It:=It2;
    End;
  End;
Begin
  If DskRoot<>NIL Then Libere(DskRoot);
  DskRoot:=NIL;
End;

Procedure EnterDiskDir;
Var S : String;
Begin
  GetDir(0,S);
  If S[1] in ['A','B'] Then DoneDskItems;
  If (DskRoot=NIL) Then InitDskItems;
End;

Procedure DiskInit;
Begin
  { Recherche le Path actuel pour le restituer en fin de programme }
  GetDir(0,DiskInitPath);
  { Les directories du lecteur actuel ne sont pas en mmoire }
  DskRoot:=NIL;
End;

Procedure DiskDone;
Var P,Ps : PLecteur;
Begin
  DoneDskItems;
  P:=Lecteurs;
  While P<>NIL do
  Begin
    Ps:=P^.Suiv;
    Dispose(P);
    P:=Ps;
  End;
  {$I-} ChDir(DiskInitPath); {$I+}
End;

{ ==== Module : Recherche de configuration ==== }

{ Info : Cette fonction recherche le type de processeur       }
{        Elle est issue de l'ouvrage --> La Bible PC          }
{        deuxime dition, et a t dveloppe par l'auteur   }
{        de ce livre Michael Tischer. La troisime dition    }
{        contient une mise  jour capable de dtecter le i486 }
{        mais n'existe qu'en .ASM et pas en inline            }

Function GetProcess : Byte;
Begin
 Inline(
        $9C/$51/$52/$57/$56/$06/$33/$C0/$50/$9D/$9C/$58/$25/$00/
        $F0/$3D/$00/$F0/$74/$13/$B2/$06/$B8/$00/$70/$50/$9D/$9C/
        $58/$25/$00/$70/$74/$36/$FE/$C2/$EB/$32/$90/$B2/$04/$B0/
        $FF/$B1/$21/$D2/$E8/$75/$12/$B2/$02/$FB/$BE/$00/$00/$B9/
        $FF/$FF/$F3/$26/$AC/$0B/$C9/$74/$02/$B2/$00/$0E/$07/$FD/
        $B0/$FB/$B9/$03/$00/$E8/$16/$00/$FA/$F3/$AA/$FC/$90/$90/
        $90/$42/$90/$FB/$88/$56/$FF/$07/$5E/$5F/$5A/$59/$9D/$EB/
        $07/$90/$5F/$83/$C7/$09/$EB/$E4
       );
End;

Procedure InstalleLecteur(No,T : Byte);
Var P,Ps : PLecteur;                  { Installe un lecteur dans le        }
Begin                                 { Systme                            }
  P:=New(PLecteur);                   { Gnre la structure PLecteur       }
  P^.Suiv:=NIL;
  P^.No:=No;                          { Charge la numero de lecteur No     }
  P^.Tpe:=T;                          { Et le type (1=Dur;2=3.5'';3=5.25'' }
  If Lecteurs=NIL                     { Met en place le chanage dans      }
  Then Lecteurs:=P                    { La liste des lecteurs              }
  Else
    Begin
      Ps:=Lecteurs;
      While (Ps^.Suiv<>NIL) do
        Ps:=Ps^.Suiv;
      Ps^.Suiv:=P;
    End;
End;

Function TrouveFormatLecteur(No : Byte) : Byte;
Begin
  If TypePC='AT'                       { Si on est sur un AT on la la        }
  Then                                 { Fonction BIOS  notre disposition   }
    Begin
      Reg.AH:=$08;                     { Demande le format du lecteur        }
      Reg.DL:=No-1;                    { Numero de lecteur de disquettes     }
      Intr($13,Reg);                   { Appelle la fonction du BIOS         }
      If Reg.BL IN [$03,$04] Then TrouveFormatLecteur:=2  { C'est un 3.5''   }
                             Else TrouveFormatLecteur:=3; { C'est un 5.25''  }
    End
  Else TrouveFormatLecteur:=2;         { Sinon, renvoie toujours 5.25''      }
End;

Procedure ConfigInit;
Var I    : Byte;
    C    : Char;
    P    : PLecteur;
    Temp : String;
Begin
  { Recherche le type de PC }
  Case Mem[$F000:$FFFE] Of
    $FC     : TypePC:='AT';
    $FB,$FE : TypePC:='XT';
    $FF     : TypePC:='PC';
  End;
  { Recherche le type de processeur }
  Processor:=NomProcess[GetProcess];
  { Recherche la version du DOS install }
  Reg.AH:=$30;                          { Demande le numero de version       }
  Intr($21,Reg);                        { Invoque le DOS                     }
  DosVer1:=Reg.AL;                      { Numero de version principal        }
  DosVer2:=Reg.AH;                      { Numero de version secondaire       }
  { Recherche les lecteurs installs sur le systme }
  For I:=1 to 26 do
  Begin
    Reg.AX:=$4408;                      { Appel IOCTL : Support amovible ?   }
    Reg.BL:=I;                          { Priphrique N I : 1=A, 2=B...    }
    Intr($21,Reg);
    If (Reg.Flags AND FCarry)=0 Then    { OK, le priphrique existe bien    }
      If Reg.AX=1                       { AX=1 : Support fixe                }
        Then InstalleLecteur(I,1)       { Disque dur                         }
        Else InstalleLecteur(I,TrouveFormatLecteur(I));  { Disquette         }
  End;
  { Recherche de la carte graphique }
  DetectGraph(GrCarte,GrMode);
  IF Not (GrCarte IN [EGA,VGA]) Then
  Begin
    BeepWrong;
    Writeln('Erreur Fatale.');
    Writeln('Carte graphique Vga ou Ega 128Ko non dtecte.');
    Writeln('Vrifiez votre quipement et redmarrez le logiciel.');
    Writeln;
    Writeln;
    Halt(1);
  End;
  If ForceEGA Then
  Begin
    GrCarte:=EGA;
    GrMode:=EGAHi;
  End;
  If ForceVGA Then
  Begin
    GrCarte:=VGA;
    GrMode:=VGAHi;
  End;
  { Recherche le chemin d'accs  Optique.EXE }
  AppPath:=ParamStr(0);
  AppPath:=FExpand(AppPath);
  FSplit(AppPath,AppPath,Temp,Temp);
End;

Procedure ConfigDone;
Begin
End;

{ === Module : Dessins sur disque ==== }

Procedure MotifsInit;
Var I,R,G,B : Integer;
    S       : TDisk;
    St      : String;
    L       : Integer;
Begin
  S.Init(AppPath+'Icones.Dat',stOpen);
  { Lit l'en-tte }
  S.Read(St,35);
  { Charge la palette de couleurs }
  For I:=0 to 15 do
  Begin
    S.Read(R,1);  S.Read(G,1);  S.Read(B,1);
    SetPalette(I,I);
    SetRGBPalette(I,R,G,B);
  End;
  { Charge les icones 20x20 }
  L:=ImageSize(1,1,20,20);
  For I:=1 to NbIco do
  Begin
    GetMem(Icones[I],L);
    S.Read(Icones[I]^,L);
  End;
  { Charge les icones 32x32 }
  L:=ImageSize(1,1,32,32);
  For I:=1 to NbIco2 do
  Begin
    GetMem(Icones2[I],L);
    S.Read(Icones2[I]^,L);
  End;
  { Charge les motifs de taille quelconque }
  For I:=1 to NbMot do
  Begin
    S.Read(MotifsC[I].X,SizeOf(Integer));
    S.Read(MotifsC[I].Y,SizeOf(Integer));
    L:=ImageSize(1,1,MotifsC[I].X,MotifsC[I].Y);
    GetMem(Motifs[I],L);
    S.Read(Motifs[I]^,L);
  End;
  { Charge les chiffres digitaux }
  L:=ImageSize(1,1,7,12);
  For I:=0 to NbChiffres do
  Begin
    GetMem(Chiffres[I],L);
    S.Read(Chiffres[I]^,L);
  End;
  { Charge les curseurs souris }
  For I:=1 to NbMouses do
    S.Read(Mouses[I],SizeOf(TMouseAspect));
  S.Done;
  { Charge les dessins de fuseaux horaires }
  S.Init(AppPath+'World.DAT',stOpen);
  S.Read(St,35);
  For I:=-12 to 12 do
  Begin
    L:=ImageSize(WorldX[I,1],1,WorldX[I,2],82);
    GetMem(World[I],L);
    S.Read(World[I]^,L);
  End;
  S.Done;
End;

Procedure MotifsDone;
Var I,L : Integer;
Begin
  L:=ImageSize(1,1,20,20);
  For I:=1 to NbIco do
    FreeMem(Icones[I],L);
  L:=ImageSize(1,1,32,32);
  For I:=1 to NbIco2 do
    FreeMem(Icones2[I],L);
  For I:=1 to NbMot do
  Begin
    L:=ImageSize(1,1,MotifsC[I].X,MotifsC[I].Y);
    FreeMem(Motifs[I],L);
  End;
  L:=ImageSize(1,1,7,12);
  For I:=0 to NbChiffres do
    FreeMem(Chiffres[I],L);
  For I:=-12 to 12 do
  Begin
    L:=ImageSize(WorldX[I,1],1,WorldX[I,2],82);
    FreeMem(World[I],L);
  End;
End;

{ ==== Module : Zoom ==== }

Procedure Zoom(X1,Y1,X2,Y2,P : Real ; G : Boolean);
Var Zoom : PZoom;
    Temp : Real;
Begin
  Zoom:=New(PZoom);
  If X1>X2 Then Begin Temp:=X1; X1:=X2; X2:=Temp; End;
  If Y1<Y2 Then Begin Temp:=Y1; Y1:=Y2; Y2:=Temp; End;
  Zoom^.X1:=X1;
  Zoom^.Y1:=Y1;
  Zoom^.X2:=X2;
  Zoom^.Y2:=Y2;
  Zoom^.Pas:=P;
  Zoom^.Gril:=G;
  Zoom^.Suiv:=Zooms;
  Zooms:=Zoom;
End;

Procedure UnZoom;
Var PZ : PZoom;
Begin
  If Zooms^.Suiv<>NIL
  Then
    Begin
      PZ:=Zooms;
      Zooms:=Zooms^.Suiv;
      Dispose(PZ);
    End
  Else BeepWrong;
End;

Function Ex(X : Real) : Integer;
Begin
  Ex:=Round((X-Zooms^.X1)*(GetMaxX-34-8)/(Zooms^.X2-Zooms^.X1)+8);
End;

Function Ey(Y : Real) : Integer;
Begin
  Ey:=Round((Y-Zooms^.Y1)*(GetMaxY-29-8)/(Zooms^.Y2-Zooms^.Y1)+8);
End;

Function Rx(X : Integer) : Real;
Var R : Real;
Begin
  R:=(X-8)*(Zooms^.X2-Zooms^.X1)/(GetMaxX-34-8)+Zooms^.X1;
  If Zooms^.Gril Then R:=Int(R/Zooms^.Pas+0.5)*Zooms^.Pas;
  Rx:=R;
End;

Function Ry(Y : Integer) : Real;
Begin
  Ry:=(Y-8)*(Zooms^.Y2-Zooms^.Y1)/(GetMaxY-29-8)+Zooms^.Y1;
End;

Procedure ZoomInit;
Begin
  Zooms:=NIL;
  Zoom(-1000,100,1000,-100,1,TRUE);
End;

Procedure ZoomDone;
Var PZ : PZoom;
Begin
  While (Zooms<>NIL) do
  Begin
    PZ:=Zooms;
    Zooms:=Zooms^.Suiv;
    Dispose(PZ);
  End;
End;

{ ==== Module : Graphique ==== }

Procedure GraphInit;
Begin
  InitGraph(GrCarte,GrMode,AppPath);
  SetTextStyle(DefaultFont,HorizDir,1);
End;

Procedure GraphDone;
Begin
  CloseGraph;
  TextMode(CO80);
End;

Procedure Outline(X,Y,L,H : Integer ; C1,C2,C3,C4 : Byte);
Begin
  SetColor(C1); MoveTo(X,  Y+H-3); LineTo(X,    Y    ); LineTo(X+L-2,Y  );
  SetColor(C2); MoveTo(X+1,Y+H-3); LineTo(X+1,  Y+1  ); LineTo(X+L-3,Y+1);
  SetColor(C3); MoveTo(X+1,Y+H-2); LineTo(X+L-2,Y+H-2); LineTo(X+L-2,Y+1);
  SetColor(C4); MoveTo(X,  Y+H-1); LineTo(X+L-1,Y+H-1); LineTo(X+L-1,Y  );
End;

{ ==== Module : Son ==== }

Procedure BeepWrong;
Begin
  Sound(800);
  Delay(50);
  Sound(600);
  Delay(70);
  Sound(800);
  Delay(50);
  NoSound;
End;

{ ==== Gestion de l'Unit Drivers ==== }

Procedure DriversInit;
Begin
  DirectVideo:=FALSE;
  ConfigInit;
  DiskInit;
  GraphInit;
  Writeln('Init : ');
  Write('Graphics motifs..... ');
  MotifsInit;
  Writeln('OK');
  Write('Souris.............. ');
  MouseInit;
  Writeln('OK');
  Write('Clavier............. ');
  KeyboardInit;
  Writeln('OK');
  Write('Zoom................ ');
  ZoomInit;
  Writeln('OK');
  Seconde:=70;
End;

Procedure DriversDone;
Begin
  ZoomDone;
  KeyboardDone;
  MotifsDone;
  MouseDone;
  GraphDone;
  DiskDone;
  ConfigDone;
End;

{ Routines MiniMaxi }

Function  MaxByte(A,B : Byte) : Byte;
Begin
  If A<B Then MaxByte:=B Else MaxByte:=A;
End;

Function  MinByte(A,B : Byte) : Byte;
Begin
  If A<B Then MinByte:=A Else MinByte:=B;
End;

Function  MaxWord(A,B : Word) : Word;
Begin
  If A<B Then MaxWord:=B Else MaxWord:=A;
End;

Function  MinWord(A,B : Word) : Word;
Begin
  If A<B Then MinWord:=A Else MinWord:=B;
End;

Function  Tan(A : Real) : Real;
Begin
  Tan:=SIN(A)/COS(A);
End;

Function  MaxReal(A,B : Real) : Real;
Begin
  If A<B Then MaxReal:=B Else MaxReal:=A;
End;

Function  MinReal(A,B : Real) : Real;
Begin
  If A<B Then MinReal:=A Else MinReal:=B;
End;

Function GetLongTime : LongInt;
Var H,M,S,S100 : Word;
Begin
  GetTime(H,M,S,S100);
  GetLongTime:=S100+S*100+M*60*100+H*60*60*100;
End;

Function UpStr(S : String) : String;
Var R : String;
    I : Byte;
Begin
  R:=S;
  For I:=1 to Length(R) do
    R[I]:=UpCase(R[I]);
  UpStr:=R;
End;

BEGIN
  ForceEGA:=FALSE;
  ForceVGA:=FALSE;
END.