{**********************************************************}
{                                                          }
{                   Projet SoftOptique                     }
{             Types Abstraits TObjet et TGroup             }
{                                                          }
{   Debut du projet : 10.04.93                   cm0=000   }
{   Mise A Jour     : 08.05.93                             }
{                                                          }
{**********************************************************}

UNIT ObjetAbs;

INTERFACE

USES Drivers;

Const
  { Constantes d'tat des objets }
     stSelected   = $0001;
     stVisible    = $0002;
     stTraite     = $0004;
  { Constantes des commandes standards }
     cmInterdite  = $8000;
     cmObjetAbs   = 000;
     cmOK         = cmObjetAbs+01;
     cmAnnule     = cmObjetAbs+02;
     cmNew        = cmObjetAbs+03;
     cmOpen       = cmObjetAbs+04;
     cmSave       = cmObjetAbs+05;
     cmSaveAs     = cmObjetAbs+09;
     cmQuit       = cmObjetAbs+06;
     cmOui        = cmObjetAbs+07;
     cmNon        = cmObjetAbs+08;
  { Constantes des codes de sortie de Exec }
     exOK         = 1;
     exAnnule     = 2;
     exOui        = 3;
     exNon        = 4;

Type PObjet   = ^TObjet;
     PGroup   = ^TGroup;
     TObjet   = Object
                  Ident       : String[10];
                  Prec,Suiv   : PObjet;
                  Owner       : PGroup;
                  Origin      : TPoint;
                  Size        : TPoint;
                  Etat        : Word;
                  BkColor     : Byte;
                  ExitCode    : Byte;
                  MouseAspect : PMouseAspect;
                  MouseInObjet: Boolean;
                  HelpCtx     : String;
                  Constructor Init(X,Y,L,H : Integer ; eBkColor : Byte);
                  Destructor  Done;                                  Virtual;
                  Procedure   HandleEvent(Var Event : TEvent);       Virtual;
                  Procedure   Draw;                                  Virtual;
                  Procedure   MakeGlobal(Src : TPoint ; Var Dst : TPoint);
                  Procedure   MakeLocal(Src : TPoint ; Var Dst : TPoint);
                  Procedure   DrawBegin;
                  Procedure   DrawEnd;
                  Procedure   AccrocheAvant(L : PObjet);
                  Procedure   AccrocheApres(L : PObjet);
                  Procedure   Decroche;
                  Procedure   SetData(Var Rec);                      Virtual;
                  Procedure   GetData(Var Rec);                      Virtual;
                  Function    DataSize : Integer;                    Virtual;
                  Procedure   GetEvent(Var Event : TEvent);          Virtual;
                  Procedure   SetEvent(Event : TEvent);              Virtual;
                  Procedure   SetCommand(Command : Word);
                  Procedure   SauveEcran;                            Virtual;
                  Procedure   RestitueEcran;                         Virtual;
                  Function    Exec : Byte;
                End;
     TGroup   = Object(TObjet)
                  Liste : PObjet;
                  Constructor Init(X,Y,L,H : Integer ; eBkColor : Byte);
                  Destructor  Done;                                  Virtual;
                  Procedure   InsertBefore(O : PObjet);
                  Procedure   Insert(O : PObjet);
                  Procedure   HandleEvent(Var Event : TEvent);       Virtual;
                  Procedure   Draw;                                  Virtual;
                  Procedure   SetData(Var Rec);                      Virtual;
                  Procedure   GetData(Var Rec);                      Virtual;
                  Function    DataSize : Integer;                    Virtual;
                End;

Var StLine       : PGroup;                  { Future Ligne d'tat            }
    Application  : PGroup;                  { Future application             }
    WrkZone      : PGroup;                  { Future Zone de Travail         }
    MouseDefaut  : PMouseAspect;            { Aspect de la souris hors-objet }
    GlobalHelp   : String;                  { Contexte d'aide courant        }

Function ExecObjet(D : PObjet) : Byte;

IMPLEMENTATION

USES Graph;

Var MouseObjet : PObjet;                    { Objet qui contient la souris   }

{ === TObjet ===}

Constructor TObjet.Init(X,Y,L,H : Integer ; eBkColor : Byte);
Begin
  Ident:='OBJET';
  Prec:=NIL;
  Suiv:=NIL;
  Owner:=NIL;
  Origin.X:=X;
  Origin.Y:=Y;
  Size.X:=L;
  Size.Y:=H;
  Etat:=0;
  BkColor:=eBkColor;
  MouseAspect:=MousePointer;
  MouseInObjet:=FALSE;
  HelpCtx:='Index';
End;

Destructor TObjet.Done;
Begin
  Decroche;
End;

Procedure TObjet.MakeGlobal(Src : TPoint ; Var Dst : TPoint);
Var O : PObjet;
Begin
  Dst:=Src;
  O:=Owner;
  While (O<>NIL) do
  Begin
    Dst.X:=Dst.X+O^.Origin.X;
    Dst.Y:=Dst.Y+O^.Origin.Y;
    O:=O^.Owner;
  End;
End;

Procedure TObjet.MakeLocal(Src : TPoint ; Var Dst : TPoint);
Var O : PObjet;
Begin
  Dst:=Src;
  O:=Owner;
  While (O<>NIL) do
  Begin
    Dst.X:=Dst.X-O^.Origin.X;
    Dst.Y:=Dst.Y-O^.Origin.Y;
    O:=O^.Owner;
  End;
End;

Procedure TObjet.HandleEvent(Var Event : TEvent);
Var P : TPoint;
Begin
  MakeGlobal(Origin,P);
  Case Event.What Of
    evMouseMove  : If (Etat AND stVisible)<>0 Then
                     If (Event.Where.X>=P.X) AND
                        (Event.Where.X<=P.X+Size.X-1) AND
                        (Event.Where.Y>=P.Y) AND
                        (Event.Where.Y<=P.Y+Size.Y-1)
                     Then
                       Begin
                         If Not MouseInObjet Then
                         Begin
                           MouseInObjet:=TRUE;
                           ChangeMouseAspect(MouseAspect);
                           MouseObjet:=@Self;
                           If HelpCtx<>'Index' Then GlobalHelp:=HelpCtx;
                         End;
                       End
                     Else
                       Begin
                         If MouseInObjet Then
                         Begin
                           MouseInObjet:=FALSE;
                           If MouseObjet=@Self Then
                           Begin
                             ChangeMouseAspect(Owner^.MouseAspect);
                             MouseObjet:=Owner;
                             GlobalHelp:=Owner^.HelpCtx;
                           End;
                         End;
                       End;
  End;
End;

Procedure TObjet.Draw;
Var P : TPoint;
Begin
  DrawBegin;
  MakeGlobal(Origin,P);
  SetFillStyle(SolidFill,BkColor);
  Bar(P.X,P.Y,P.X+Size.X-1,P.Y+Size.Y-1);
  Etat:=Etat OR stVisible;
  DrawEnd;
End;

Procedure TObjet.AccrocheAvant(L : PObjet);
Var O : PObjet;
Begin
  If (L<>NIL) Then
  Begin
    O:=L;                                   { Cherche le dbut de la liste }
    While (O^.Prec<>NIL) do O:=O^.Prec;     { Pour ajouter TObjet au dbut }
    Suiv:=O;
    Prec:=NIL;
    O^.Prec:=@Self;
  End;
End;

Procedure TObjet.AccrocheApres(L : PObjet);
Var O : PObjet;
Begin
  If (L<>NIL) Then
  Begin
    O:=L;
    While (O^.Suiv<>NIL) do O:=O^.Suiv;
    Prec:=O;
    Suiv:=NIL;
    O^.Suiv:=@Self;
  End;
End;

Procedure TObjet.Decroche;
Begin
  If Owner<>NIL Then
    If Owner^.Liste=@Self Then Owner^.Liste:=Suiv;
  if (Suiv<>NIL) Then Suiv^.Prec:=Prec;
  if (Prec<>NIL) Then Prec^.Suiv:=Suiv;
  Prec:=NIL;
  Suiv:=NIL;
  Owner:=NIL;
End;

Procedure TObjet.SetData(Var Rec);
Begin
End;

Procedure TObjet.GetData(Var Rec);
Begin
End;

Function TObjet.DataSize : Integer;
Begin
  DataSize:=0;
End;

Procedure TObjet.GetEvent(Var Event : TEvent);
Begin
  If Owner<>NIL Then Owner^.GetEvent(Event);
End;

Procedure TObjet.SetEvent(Event : TEvent);
Begin
  If Owner<>NIL Then Owner^.SetEvent(Event);
End;

Procedure TObjet.SetCommand(Command : Word);
Var Event : TEvent;
Begin
  Event.What:=evCommand;
  Event.Command:=Command;
  SetEvent(Event);
End;

Procedure TObjet.SauveEcran;
Begin
End;

Procedure TObjet.RestitueEcran;
Begin
End;

Function TObjet.Exec : Byte;
Var Event : TEvent;
Begin
  ExitCode:=0;
  SauveEcran;
  Draw;
  Event.What:=evMouseMove;
  Event.Where.X:=MouseX;
  Event.Where.Y:=MouseY;
  HandleEvent(Event);
  Repeat
    GetEvent(Event);
    if Event.What<>evNothing Then
    Begin
      StLine^.HandleEvent(Event);
      HandleEvent(Event);
    End;
  Until ExitCode<>0;
  RestitueEcran;
  Exec:=ExitCode;
End;

Procedure TObjet.DrawBegin;
Var P : TPoint;
Begin
  MakeGlobal(Origin,P);
  MouseClipON(P.X,P.Y,P.X+Size.X-1,P.Y+Size.Y-1);
End;

Procedure TObjet.DrawEnd;
Begin
  MouseClipOFF;
End;

{ === TGroup ==}

Constructor TGroup.Init(X,Y,L,H : Integer ; eBkColor : Byte);
Begin
  inherited Init(X,Y,L,H,eBkColor);
  Ident:='GROUP';
  Liste:=NIL;
End;

Destructor TGroup.Done;
Var O,O2 : PObjet;
Begin
  O:=Liste;
  While (O<>NIL) do
  Begin
    O2:=O^.Suiv;
    Dispose(O,Done);
    O:=O2;
  End;
  inherited Done;
End;

Procedure TGroup.InsertBefore(O : PObjet);
Var O2 : PObjet;
Begin
  O2:=O;
  While (O2<>NIL) do
  Begin
    O2^.Owner:=@Self;
    O2:=O2^.Suiv;
  End;
  if (Liste=NIL) Then Liste:=O
                 Else Begin
                        O^.AccrocheAvant(Liste);
                        Liste:=O;
                      End;
End;

Procedure TGroup.Insert(O : PObjet);
Var O2 : PObjet;
Begin
  O2:=O;
  While (O2<>NIL) do
  Begin
    O2^.Owner:=@Self;
    O2:=O2^.Suiv;
  End;
  if (Liste=NIL) Then Liste:=O
                 Else O^.AccrocheApres(Liste);
End;

Procedure TGroup.Draw;
Var O : PObjet;
    P : TPoint;
Begin
  DrawBegin;
  inherited Draw;
  MakeGlobal(Origin,P);
  O:=Liste;
  While (O<>NIL) do
  Begin
    O^.Draw;
    O:=O^.Suiv;
  End;
  DrawEnd;
End;

Procedure TGroup.HandleEvent(Var Event : TEvent);
Var O : PObjet;
Begin
  inherited HandleEvent(Event);
  O:=Liste;
  While ((O<>NIL) And (Event.What<>evNothing)) do
  Begin
    O^.HandleEvent(Event);
    O:=O^.Suiv;
  End;
End;

Type Tableau = Array[0..60000] Of Byte;

Procedure TGroup.SetData(Var Rec);
Var Pos : Integer;
    O   : PObjet;
    Tbl : ^Tableau;
Begin
  Tbl:=@Rec;
  Pos:=0;
  O:=Liste;
  While (O<>NIL) do
  Begin
    O^.SetData(Tbl^[Pos]);
    Pos:=Pos+O^.DataSize;
    O:=O^.Suiv;
  End;
End;

Procedure TGroup.GetData(Var Rec);
Var Pos : Integer;
    O   : PObjet;
    Tbl : ^Tableau;
Begin
  Tbl:=@Rec;
  Pos:=0;
  O:=Liste;
  While (O<>NIL) do
  Begin
    O^.GetData(Tbl^[Pos]);
    Pos:=Pos+O^.DataSize;
    O:=O^.Suiv;
  End;
End;

Function TGroup.DataSize : Integer;
Var Result : Integer;
    O      : PObjet;
Begin
  Result:=0;
  O:=Liste;
  While (O<>NIL) do
  Begin
    Result:=Result+O^.DataSize;
    O:=O^.Suiv;
  End;
  DataSize:=Result;
End;

Function ExecObjet(D : PObjet) : Byte;
Var R : Byte;
Begin
  Application^.Insert(D);
  R:=D^.Exec;
  Dispose(D,Done);
  ExecObjet:=R;
End;

BEGIN
  MouseObjet:=NIL;
  MouseDefaut:=MousePointer;
END.